require, "Eric/fft_utils.i";
require, "~eric/work/i/bsp/bispectrum.i";

func cp40_bisp(id, scale=, win=, nsp=)
{
  cp40_cancel_task, id, "*";
  if (is_void(scale)) scale = cp40_scale(id);
  if (is_void(win)) win=[83,343,6,281];
  cp40_config, id, scale=scale, correction="weighted", exposure=1, window=win;
  cp40_rewind, id;
  cp40_schedule_task, id, "it0", "it1";

  
  win0 = cp40_task_limits(id, "it0");
  win1 = cp40_task_limits(id, "it1");
  if (anyof(win1 != win0)) error, "bad window";
  win = win0;
  
  
  nx = win(2) - win(1) + 1;
  ny = win(4) - win(3) + 1;
  dim = fft_best_dim(max(nx, ny));
  write, "DIM=",dim;

  if (is_void(nsp) || nsp==3) {
    gen = [1, 2, dim+1];
  } else if (nsp == 5) {
    gen = [1, 2, dim+1, dim+2, 2*dim]
  } else if (nsp == 7) {
    gen = [1, 2, dim+1, dim+2, 2*dim, 3, 2*dim+1]
  } else if (nsp == 11) {
    gen = [1, 2, dim+1, dim+2, 2*dim, 3, 2*dim+1, dim+3, 2*dim+2, 3*dim,
          2*dim-1];
  } else error, "bad NSP should be 3, 5, 7 or 11";
  
  valid = fft_dist(dim, dim) < (dim-1)/2 + 0.5;
  bisp = bsp_new_sum(valid, generator=gen, hermitian=1);

  ws = fft_setup([2,dim,dim]);
  
  maxfailures = 10;
  nfailures = 0;
  for (;;) {
    //if (catch(0x08)) {
    //  cp40_clear_task, id, "*";
    //  write, catch_message;
    //  if (++nfailures >= maxfailures) break;
    //}
    count = cp40_run_tasks(id, 1);
    if (count != 1) {
      if (++nfailures >= maxfailures) break;
      continue;
    }
    nfailures = 0;
    (z1 = array(complex, dim, dim))(1:nx, 1:ny) = cp40_task_data(id, "it0");
    (z2 = array(complex, dim, dim))(1:nx, 1:ny) = cp40_task_data(id, "it1");
    fft_inplace, z1, -1, setup=ws;
    fft_inplace, z2, -1, setup=ws;
    bsp_integ2, bisp, z1, z2;
    cp40_clear_task, id, "*";
    write, format=" %d", bisp.count;
  }

  return bisp;
}

func cp40_all_its(id, cancel=, correction=, exposure=)
{
  tasks= "it"+["0","1","2","3"];
  if (! am_subroutine()) {
    return cp40_integ(id, tasks, cancel=cancel, correction=correction,
                      exposure=exposure);
  }
  cp40_integ, id, tasks, cancel=cancel, correction=correction,
    exposure=exposure;
}

func cp40_all_ics(id, cancel=, correction=, exposure=)
{
  tasks= "ic"+["01","02","03","12", "13", "23"];
  if (! am_subroutine()) {
    return cp40_integ(id, tasks, cancel=cancel, correction=correction,
                      exposure=exposure);
  }
  cp40_integ, id, tasks, cancel=cancel, correction=correction,
    exposure=exposure;
}

func cp40_fit_gauss(x=, y=, z=, weight=, param=, fwhm=, sigma=)
{
  if ((nparams= numberof(param)) < 2 || nparams > 4) {
    error, "bad number of elements in PARAM";
  }
  if (nparams <= 3) {
    /* Width of gaussian is required. */
    if (is_void(sigma) == is_void(fwhm))
      error, "exactly one of SIGMA or FWHM is required";

    if (nparams==2) op= _cp40_fit_gauss_2;
    else            op= _cp40_fit_gauss_3;
  } else {
    if (! is_void(sigma) || ! is_void(fwhm))
      error, "SIGMA and FWHM must be unspecified";
    op= _cp40_fit_gauss_4;
  }

}

func _cp40_fit_gauss_3(unused, a, &jac, deriv=)
/* DOCUMENT _cp40_fit_gauss_3
   
   Parameters:
     A(1) = intensity of spike
     A(2) = abscissa of spike
     A(3) = ordinate of spike
*/
{
  extern _cp40_fit_gauss_x, _cp40_fit_gauss_y, _cp40_fit_gauss_scale;
  u= _cp40_fit_gauss_scale*(_cp40_fit_gauss_x - a(2));
  v= _cp40_fit_gauss_scale*(_cp40_fit_gauss_y - a(3));
  e= exp(-u*u-v*v);
  if (deriv) {
    se= (2.0*_cp40_fit_gauss_scale*a(1))*e;
    jac= [e(*), (u*se)(*), (v*se)(*)];
  }
  return a(1)*e;
}

func _cp40_fit_gauss_4(unused, a, &jac, deriv=)
/* DOCUMENT _cp40_fit_gauss_4
   
   Parameters:
     A(1) = intensity of spike
     A(2) = abscissa of spike
     A(3) = ordinate of spike
     A(4) = standard deviation of spike
*/
{
  extern _cp40_fit_gauss_x, _cp40_fit_gauss_y;
  s= sqrt(0.5)/a(4);
  u= s*(_cp40_fit_gauss_x - a(2));
  v= s*(_cp40_fit_gauss_y - a(3));
  u2v2= u*u + v*v;
  e= exp(-u2v2);
  if (deriv) {
    se= (2.0*s*a(1))*e;
    jac= [e(*), (u*se)(*), (v*se)(*), ((2.0*a(1)/a(4))*u2v2*e)(*)];
  }
  return a(1)*e;
}
func _cp40_fit_gauss_5(unused, a, &jac, deriv=)
/* DOCUMENT _cp40_fit_gauss_5
   
   Parameters:
     A(1) = intensity of spike
     A(2) = abscissa of spike
     A(3) = ordinate of spike
     A(4) = 1/sqrt(2)/SIGMA = sqrt(log(16))/FWHM

     FWHM = sqrt(log(16))/A(4)
     Var{FWHM} ~ log(16)/A(4)^4 * Var{A(4)}

     SIGMA = 1/sqrt(2)/A(4)
     Var{SIGMA} ~ 0.5/A(4)^4 * Var{A(4)}
     
*/
{
  return;
}

/*---------------------------------------------------------------------------*/

func cp40_measure(id, task, chunk)
/* DOCUMENT cp40_measure(id, task)
       -or- cp40_measure(id, task, chunk)
     Measure mean photon counting estimator given by TASK, and its estimated
     standard deviation, from _all_ data of instance ID.  The result is a 3D
     array:
       cp40_measure(id, task)(..,1)= estimator
       cp40_measure(id, task)(..,2)= standard_deviation_of_estimator
     The routine proceed by measuring the estimator for disjoint pieces of
     CHUNK frames (default 100).

   SEE ALSO cp40_schedule_task, cp40_run_tasks. */
{
  /* Notes: Let Y = (X_1 + ... + X_N) / N   where the X_i are N independent
   *   measurements of the same random variable with mean E(X) and variance
   *   Var(X).  Then:
   *            E(Y) = E(X)
   *   and    Var(Y) = Var(X)/N */

  if (numberof(cp40_list_of_tasks(id))) error, "there are scheduled tasks";
  cp40_rewind, id;
  cp40_schedule_task, id, task;
  if (is_void(chunk)) chunk= 100;

  sx= sx2= 0.0;
  nframes= 0;
  nchunks= 0;
  n= 0;
  cp40_clear_task, id, task;
  do {
    dn= cp40_run_tasks(id, chunk-n);
    if ((n+=dn) == chunk) {
      x= double(cp40_task_data(id, task))/n;
      sx+= x;
      sx2+= x*x;
      nframes+= n;
      nchunks+= 1;
      n= 0;
      cp40_clear_task, id, task;
    }
  } while (dn);
  l= cp40_task_limits(id, task);
  cp40_cancel_task, id, "*";
  x= sx/nchunks;
  sx= array(double, l(1):l(2), l(3):l(4), 2);
  sx(..,1)= x;
  sx(..,2)= sqrt((sx2/nchunks - x*x) / (nchunks - 1));
  return sx;
}

func compare_4(a,ascl,b,bscl)
{
  as= double(ascl);
  bs= double(bscl);
  stat,
    *a(1)/as - *b(1)/bs, *a(2)/as - *b(2)/bs,
    *a(3)/as - *b(3)/bs, *a(4)/as - *b(4)/bs; 
}

