#cp40save & cp40send 3457 & cp40ctrl 3456
namespace eval ::rt {
  variable Priv
}
proc ::rt::Init {} {
  variable Priv
  foreach {key value} {
    data,host   "cp40"
    data,port    3457
    fov,refresh    30
    task,refresh  500
  } {
    if {! [info exists Priv($key)]} {
      set Priv($key) $value
    }
  }

  wm withdraw .
  set this [instance]
  set config_file "cp40rtd.attr"
  if {[file exists $config_file]} {
    puts stderr "reading configuration from \"$config_file\""
    if {[catch {$this restore_config -preprocessing $config_file} msg]} {
      puts stderr "WARNING: $msg"
    }
  }
  $this command ::rt::callback
  catch {connect}
  $this schedule "fov" "ic01" "ic23"
  mainWindow
}

#------------------------------------------------------------------------------
# instance --
# Return current real-time CP40 instance, if none exists, a new one
# is created.
#
# Arguments:
# (none)
proc ::rt::instance {} {
  variable Priv
  if {[info exists Priv(instance)] && [cp40 exists $Priv(instance)]} {
    return $Priv(instance)
  }
  set Priv(instance) [cp40 new "#auto"]
}

# disconnect --
# Disconnect real-time CP40 instance from photon server.
#
# Arguments:
# (none)
proc ::rt::disconnect {} {
  [instance] close
}

# connect --
# Connect or reconnect real-time instance to photon server.
#
# Arguments:
# (none)
proc ::rt::connect {} {
  variable Priv
  set this [instance]
  if {[lindex [$this assoc] 0] != "-socket"} {
    $this assoc -socket $Priv(data,host) $Priv(data,port)
  }
}

# duration --
# Set/query number of frames used in real-time averaging.
proc ::rt::duration {{nframes {}}} {
  variable Priv
  set this [instance]
  if {$nframes == ""} {
    return [$this stat duration]
  }
  set nframes [$this stat duration $nframes]
  set Priv(fov,refresh) [expr ($nframes+1)/2]
  return $nframes
}

# clear --
# Clear scheduled task.
#
# Arguments:
# args -	Identifiers of tasks to clear, can be "all".
proc ::rt::clear args {
  set this [instance]
  set allTasks [$this tasks all]
  foreach task $args {
    if {$task == "all"} {
      foreach task $allTasks {
	set taskArray($task) ""
      }
    } elseif {[lsearch -exact $allTasks $task] >= 0} {
      set taskArray($task) ""
    }
  }
  foreach task [array names taskArray] {
    $this link x $task
    @inplace {$x[]=0}
  }
}

#------------------------------------------------------------------------------
# callback --
# Callback procedure called when new frames where read.
#
# Arguments:
# (none)
proc ::rt::callback {} {
  variable Priv
  set this [instance]

  # Figure out which tasks should be refreshed and refresh them.
  set new [$this stat elapsed frames]
  set taskList [$this tasks running]
  if {[info exists Priv(elapsed,frames)]} {
    set old $Priv(elapsed,frames)
    if {! [needRefresh $old $new $Priv(task,refresh)]} {
      if {[needRefresh $old $new $Priv(fov,refresh)] \
	  && [lsearch -exact $taskList "fov"] >= 0} {
	set taskList "fov"
      } else {
	set taskList {}
      }
    }
  }
  set Priv(elapsed,frames) $new
  set imageList [image names]
  foreach task $taskList {
    if {$task == "fov"} {
      if {[lsearch -exact $imageList $task] >= 0} {
	$task config -data $this->$task
      }
      # updateStatistics
    } else {
      refreshView $task
    }
  }
}

# needRefresh --
# Return non-zero if counter, rounded down to GRAIN, changes by at
# least one GRAIN from OLD to NEW.  
#
# Arguments:
# old -		Old counter value.
# new -		New counter value.
# grain -	Refresh grain.
proc ::rt::needRefresh {old new grain} {
  expr floor($new/double($grain)) > floor($old/double($grain))
}

#------------------------------------------------------------------------------
proc ::rt::startFPS {} {
  cancelFPS
  UpdateFPS
}
proc ::rt::UpdateFPS {} {
  variable Priv
  set clicks [clock clicks]
  set frames [[instance] stat elapsed frames]
  if {[info exists Priv(fps,clicks)]} {
    # Assume 1e6 clicks/second.
    set lag [expr ($clicks - $Priv(fps,clicks)) / 1e6];
    set Priv(fps) [format "%6.1f" [expr ($frames - $Priv(fps,frames)) / $lag]]
  }
  set Priv(fps,frames) $frames
  set Priv(fps,clicks) $clicks
  set Priv(fps,after) [after 60000 rt::UpdateFPS]
}

proc ::rt::cancelFPS {} {
  variable Priv
  if {[info exists Priv(fps,after)]} {
    catch {after cancel $Priv(fps,after)}
    unset Priv(fps,after)
  }
}

#------------------------------------------------------------------------------
proc ::rt::CreateTopLevel {w title} {
  if {[winfo exists $w]} {
    wm deiconify $w
    return 0
  } else {
    toplevel $w
    wm title $w $title
    wm iconname $w $title
    return 1
  }
}

proc ::rt::viewWindow task {
  variable Priv
  set top .rt_$task
  if {! [CreateTopLevel $top "View: $task"]} {
    return $top
  }
  set plot $top.plot
  newPlot $task $plot -maxcolor 256 -mincolor 256
  bind $plot <Configure> "+::rt::refreshView $plot 1"
  pack $plot -fill both -expand 1
  pgx::loadct /home/eric/Gist/colors1.tbl {Mac Style}
}

proc ::rt::mainWindow {} {
  variable Priv
  set top .rt_main
  if {! [CreateTopLevel $top "SPID/CP40 Real Time Display"]} {
    return $top
  }

  # Tool bar.
  set bar $top.bar
  frame $bar -bd 1 -relief raised
  pack $bar -fill x -expand 0

  # Task menu-button.
  set button $bar.tasks
  set menu $button.menu
  menubutton $button -text "Tasks" -menu $menu -relief raised
  menu $menu
  set prev ""
  foreach task [list it0 it1 it2 it3 ic01 ic23] {
    if {$prev != "" && [string compare \
	[string range $task 0 1] \
	[string range $prev 0 1]]} {
      $menu add separator
    }
    $menu add cascade -label $task -menu $menu.$task
    set m [menu $menu.$task]
    foreach {label cmd} {
      "start"      start
      "stop"       stop
      "cancel"     cancel
      "view/hide"  viewHide
      "clear"      clear
      "save as..." saveAs
    } {
      $m add command -label $label -command "::rt::$cmd $task"
    } 
  }
  pack $button -side left -expand 0

  # Frame/rate.
  startFPS
  pack [label $bar.fps1 -relief sunken -background White \
      -textvariable ::rt::Priv(fps)] -side right -expand 0
  pack [label $bar.fps2 -text "FPS: "] -side right -expand 0

  # Field-of-view image.
  set w $top.fov
  if {[lsearch -exact [image names] "fov"] < 0} {
    image create photo "fov" -format "cp40"
  }
  label $w -image "fov"
  pack $w -side top -expand 0

  # Flux/spread plots.
  set row 1
  foreach plot {flux spread} {
    set w $top.$plot
    newPlot $plot $w -maxcolor 256 -mincolor 256
    bind $w <Configure> "+::rt::refreshCurve $plot 1"
    pack $w -side top -expand 1 -fill both
    incr row
  }
}

proc ::rt::newPlot {name w args} {
  variable Priv
  if {[selectPlot $name]} {
    return -code error "plot $name already exists"
  }
  eval pgplot $w $args
  set Priv(plot,$name) [pg::open $w/xtk]
  bind $w <Destroy> "+::rt::closePlot $name"
  pg::ask 0
  return $w
}

proc ::rt::selectPlot name {
  variable Priv
  if {[info exists Priv(plot,$name)]} {
    set id $Priv(plot,$name)
    pg::slct $id
    if {[pg::qid] == $id} {
      return 1
    }
    unset Priv(plot,$name)
  }
  return 0
}

proc ::rt::closePlot name {
  if {[selectPlot $name]} {
    variable Priv
    pg::clos
    unset Priv(plot,$name)
  }
}

proc ::rt::refreshCurve {name {redraw 1}} {
  variable Priv
  if {! [selectPlot $name]} return
  set xmin 0
  set xmax 500
  set ymin 0
  set ymax 100
  set just 0
  set axis 2
  pg::bbuf
  if {$redraw} {
    pg::page
  }
  pg::sch 1.4142
  pgx::color cyan
  pg::env $xmin $xmax $ymin $ymax $just $axis
  #pg::sch 2.0
  pg::lab "time (mn)" $name [string toupper $name]
  pg::sch 1.0
  pg::ebuf
}

proc ::rt::refreshView {task {redraw 0}} {
  variable Priv
  if {! [selectPlot $task]} return
  set this [instance]
  if {[lsearch -exact [$this tasks all] $task] < 0} return
  $this link z $task
  set xmin -0.5
  set xmax [@expr {dimsof($z)[0] - 0.5}]
  set ymin -0.5
  set ymax [@expr {dimsof($z)[0] - 0.5}]
  set just 1
  set axis 0
  pg::bbuf
  if {$redraw} {
    pg::page
  }
  pg::sch 1.0
  pgx::color cyan
  pg::svp 0.05 0.95 0.05 0.95
  pg::wnad $xmin $xmax $ymin $ymax
  #pg::env $xmin $xmax $ymin $ymax $just $axis
  #pg::lab "X-axis (pixels)" "Y-axis (pixels)" [string toupper $task]
  set zmax [@expr max($z)]
  set zmin [@expr min($z)]
  set cir [pg::qcir]
  set cmin [lindex $cir 0]
  set cmax [lindex $cir 1]
  if {$zmax > $zmin && $cmax > $cmin} {
    if {$zmin > 0} {
      @set z $z-$zmin	;# do not use @inplace because Z is a link
    }
    pg::pixl long($cmin+0.499999+double($cmax-$cmin)/double($zmax-$zmin)*$z) \
	$xmin $xmax $ymin $ymax
  }
  set xopt "BCNMST"; set yopt "BCNMST"
  set xtick 0.0;     set ytick 0.0
  set nxsub 0;       set nysub 0
  pg::box $xopt $xtick $nxsub $yopt $ytick $nysub
  pg::ebuf
}

#------------------------------------------------------------------------------

proc ::rt::start task {
  [instance] schedule $task
}

proc ::rt::stop task {
  [instance] stop $task
}

proc ::rt::cancel task {
  [instance] cancel $task
}

proc ::rt::viewHide task {
  set w .rt_$task
  if {[winfo exists $w]} {
    if {[wm state $w] == "normal"} {
      wm withdraw $w
    } else {
      wm deiconify $w
    }
  } else {
    viewWindow $task
  }
}

proc ::rt::saveAs task {
  puts stderr "save as..."
}

#------------------------------------------------------------------------------
# saveImage --
#	Save task result as FITS images.
proc ::rt::saveImage {args} {
  set this [instance]
  set prefix [clock format [clock seconds] \
      -format "cp40-%Y:%j:%H:%M:%S" -gmt 1]
  if {[llength $args] == 0} {
    set args "all"
  }
  foreach task $args {
    $this saveImage -prefix -- $task $prefix
  }
}

#------------------------------------------------------------------------------

proc ::rt::calibrateClicks {{ms 100}} {
  set u [clock clicks]
  after $ms
  expr double([clock clicks] - $u)/$ms
}


proc ::rt::statistics {} {
  variable Priv

  # 
  set keylist [list flux nframes spread x y]
  set chnlist [list 0 1 2 3]
  if {! [info exists Priv(stat,count)]} {
    set Priv(stat,start) [expr [clock seconds] - \
	[$Priv(instance) stat elapsed seconds]]
    set Priv(stat,count)    0
    set Priv(stat,size)  1000
    foreach key $keylist { 
      @set Priv(stat,$key) array(float,4,$Priv(stat,size)) 
    }
    @set Priv(stat,elapsed) array(float,$Priv(stat,size))
  }
  set i $Priv(stat,count)
  if {$i >= $Priv(stat,size)} {
    # Augment size of buffers.
    set newsize [expr 2*$Priv(stat,size)]
    @set tmp array(float,4,$newsize)
    set j [expr $i - 1]
    foreach key $keylist {
      @inplace {$tmp[,:$j]= $Priv(stat,$key)}
      @set Priv(stat,$key) $tmp
    }
    @set tmp array(float,4,$newsize)
    @inplace {$tmp[:$j]= $Priv(stat,elapsed)}
    @set Priv(stat,elapsed) $tmp
    unset tmp
    set Priv(stat,size) $newsize
  }

  # Store statistics information.
  $Priv(instance) stat average avg 
  set elapsed [$Priv(instance) stat elapsed seconds]
  foreach key $keylist {
    foreach chn $chnlist {
      @inplace {$Priv(stat,$key)[$chn,$i]= $avg($key,$chn)}
    }
  }
  @inplace {$Priv(stat,elapsed)[$i]= $elapsed}
  incr Priv(stat,count)

  set symbol 7
  pg::pt1 $x $y $symbol
}

#------------------------------------------------------------------------------
# PGPLOT EXTENSIONS
namespace eval ::pgx {
  variable ColorList [list white black red green blue cyan magenta yellow \
      orange]
}

proc ::pgx::color value {
  variable ColorList
  if {[catch {incr value 0}]} {
    set value [lsearch -exact $ColorList [string tolower $value]]
    if {$value < 0} {
      return -code error "unknown color \"$value\""
    }
  }
  ::pg::sci $value
}

proc ::pgx::loadct {file {which __nil__}} {
  # Open color table file.
  set chn [open $file]

  # Read number of color tables.
  @read number $chn byte
  set number [@expr $number]
  if {[file size $file] != 1 + (3 * 256 + 32) * $number} {
    close $chn
    return -code error "bad file size ($file)"
  }

  if {[catch {incr which 0}]} {
    # Read names of color tables.
    set nameList {}
    seek $chn [expr 1 + 3 * 256 * $number]
    for {set index 0} {$index < $number} {incr index} {
      scan [read $chn 32] "%\[^\n\]" name
      lappend nameList [string trim $name]
    }
    if {$which == "__nil__"} {
      close $chn
      return $nameList
    }
    set index [lsearch -exact [string tolower $nameList] \
	[string tolower $which]]
  } else {
    set index $which
  }
  if {$index < 0 || $index >= $number} {
    close $chn
    return -code error "bad color table name/index \"$which\""
  }

  # Read color tables data and close color table file.
  seek $chn [expr 1 + 768 * $index]
  @read rgb $chn byte 256 3
  close $chn
  @set rgb float($rgb/255.0)

  # Set color table.
  set cir [pg::qcir]
  set cmin [lindex $cir 0]
  set cmax [lindex $cir 1]
  set a [expr 255.0 / ($cmax - $cmin)]
  set b [expr 0.499999 - $cmin * $a]
  for {set c $cmin} {$c <= $cmax} {incr c} {
    set i [expr int($a*$c + $b)]
    ::pg::scr $c {$rgb[$i,0]} {$rgb[$i,1]} {$rgb[$i,2]}
  }
}
#------------------------------------------------------------------------------
::rt::Init	;# MUST BE LAST COMMAND
#------------------------------------------------------------------------------