#ifndef _GPL_MATH_H
#define _GPL_MATH_H 1

#include "gpl_base.h"
#include <math.h>

GPL_BEGIN_DECLS

/* Some constants. */
#define GPL_PI_D    3.14159265358979323846264338327950
#define GPL_PI_F    3.14159265358979323846264338327950f
#define GPL_EXP1_D  2.71828182845904523536028747135266
#define GPL_EXP1_F  2.71828182845904523536028747135266f
#define GPL_SQRT2_D 1.41421356237309504880168872420960
#define GPL_SQRT2_F 1.41421356237309504880168872420960f
#define GPL_D2R_D   0.0174532925199432957692369076848861  /* degree -> radian */
#define GPL_D2R_F   0.0174532925199432957692369076848861f /* degree -> radian */

/* Definitely in standard math library: */
extern double exp(double);
extern double log(double);
extern double sqrt(double);
extern double sin(double);
extern double cos(double);
extern double tan(double);
extern double asin(double);
extern double acos(double);
extern double atan(double);
extern double atan2(double, double);
extern double sinh(double);
extern double cosh(double);
extern double tanh(double);
extern double floor(double);
extern double ceil(double);
extern double rint(double value);

/* May be in standard math library, maybe not: */
extern double hypot(double x, double y);
#ifdef SOLARIS
extern int isinf(double value);
extern int isnan(double value);
#endif /* SOLARIS */
extern int finite(double value);

/* Define HAVE_SINF if you have single precision math functions. */
#ifdef HAVE_SINF
extern float sqrtf(float);
extern float expf(float);
extern float logf(float);
extern float sinf(float);
extern float cosf(float);
extern float tanf(float);
extern float asinf(float);
extern float acosf(float);
extern float atanf(float);
extern float atan2f(float, float);
extern float sinhf(float);
extern float coshf(float);
extern float tanhf(float);
#else
# define expf(x)      exp((double)(x))
# define logf(x)      log((double)(x))
# define log10f(x)    log10((double)(x))
# define log2f(x)     log2((double)(x))
# define sqrtf(x)     sqrt((double)(x))
# define cbrtf(x)     cbrt((double)(x))
# define sinf(x)      sin((double)(x))
# define cosf(x)      cos((double)(x))
# define tanf(x)      tan((double)(x))
# define asinf(x)     asin((double)(x))
# define acosf(x)     acos((double)(x))
# define atanf(x)     atan((double)(x))
# define atan2f(y, x) atan2((double)(y), (double)(x))
# define sinhf(x)     sin((double)(x))
# define coshf(x)     cos((double)(x))
# define tanhf(x)     tan((double)(x))
#endif

/*---------------------------------------------------------------------------*/
/* MATH FUNCTIONS */

/* NOTE: All complex function algorithms work when the destination
   array (Z) is identical to the source array(s) (X).  */

extern void gpl_sqrtc(float z[2], float x[2]);
extern void gpl_expc(float z[2], float x[2]);
extern void gpl_logc(float z[2], float x[2]);
extern void gpl_powcc(float z[2], float x[2], float y[2]);
extern int gpl_powii(int b, int p);
extern float gpl_powfi(float b, int p);
extern void gpl_powci(float z[2], float b[2], int p);
extern void gpl_sinc(float z[2], float x[2]);
extern void gpl_cosc(float z[2], float x[2]);
extern void gpl_tanc(float z[2], float x[2]);
extern void gpl_asinc(float z[2], float x[2]);
extern void gpl_acosc(float z[2], float x[2]);
extern void gpl_atanc(float z[2], float x[2]);
extern void gpl_sinhc(float z[2], float x[2]);
extern void gpl_coshc(float z[2], float x[2]);
extern void gpl_tanhc(float z[2], float x[2]);
extern void gpl_signc(float z[2], float x[2]);
extern float gpl_hypotf(float x, float y);
extern void gpl_squarec(float z[2], float x[2]);
extern void gpl_cubec(float z[2], float x[2]);

extern void gpl_sqrtz(double z[2], double x[2]);
extern void gpl_expz(double z[2], double x[2]);
extern void gpl_logz(double z[2], double x[2]);
extern void gpl_powzz(double z[2], double x[2], double y[2]);
extern double gpl_powdi(double b, int p);
extern void gpl_powzi(double z[2], double b[2], int p);
extern void gpl_sinz(double z[2], double x[2]);
extern void gpl_cosz(double z[2], double x[2]);
extern void gpl_tanz(double z[2], double x[2]);
extern void gpl_asinz(double z[2], double x[2]);
extern void gpl_acosz(double z[2], double x[2]);
extern void gpl_atanz(double z[2], double x[2]);
extern void gpl_sinhz(double z[2], double x[2]);
extern void gpl_coshz(double z[2], double x[2]);
extern void gpl_tanhz(double z[2], double x[2]);
extern void gpl_signz(double z[2], double x[2]);
extern double gpl_hypotd(double x, double y);
extern void gpl_squarez(double z[2], double x[2]);
extern void gpl_cubez(double z[2], double x[2]);

/*---------------------------------------------------------------------------*/
GPL_END_DECLS
#endif /* _GPL_MATH_H */
