/*
 * gpl_num_priv.h --
 *
 *	(Re)definitions for compiling Slatec routines in GPLib.  To summarize,
 *	according to the current definition of GPL_SLATEC_SINGLE_PRECISION
 *	or GPL_SLATEC_DOUBLE_PRECISION, this header file defines macros for
 *	data types, routines names, ...  Using the preprocessor, it is
 *	therefore possible to avoid duplication of code for single/double
 *	precision version of your routines.
 *
 *	Typical usage is:                 */
#if 0 /* <--(protect the following sample) */

#ifndef _THIS_C
#define _THIS_C 1

#include "some_header.h"
#include "some_other_header.h"

#define GPL_NUM_SINGLE_PRECISION
#include "gpl_num_priv.h"
#define SOME_FUNCTION         some_name_single
#define SOME_OTHER_FUNCTION   some_other_name_single
#include __FILE__
#undef GPL_NUM_SINGLE_PRECISION

#define GPL_NUM_DOUBLE_PRECISION
#include "gpl_num_priv.h"
#define SOME_FUNCTION         some_name_double
#define SOME_OTHER_FUNCTION   some_other_name_double
#include __FILE__
#undef GPL_NUM_DOUBLE_PRECISION

#else /* _THIS_C defined */

int SOME_FUNCTION(some_type_t arg, ...)
{
  /* implementation of SOME_FUNCTION */
}

void SOME_OTHER_FUNCTION(some_other_type_t arg, ...)
{
  /* implementation of SOME_OTHER_FUNCTION */
}


#undef SOME_FUNCTION
#undef SOME_OTHER_FUNCTION
#endif /* _THIS_C */

#endif /* <--(end of sample) */
/*---------------------------------------------------------------------------*/
#if ! (( defined(GPL_NUM_SINGLE_PRECISION) && \
        !defined(GPL_NUM_DOUBLE_PRECISION)) || \
       (!defined(GPL_NUM_SINGLE_PRECISION) && \
         defined(GPL_NUM_DOUBLE_PRECISION)))
# error "exactly one of GPL_NUM_SINGLE_PRECISION or GPL_NUM_DOUBLE_PRECISION"
#endif

#ifndef USE_CONST_VAR_FOR_CONSTANTS
# define USE_CONST_VAR_FOR_CONSTANTS 0
#endif

#undef TRUE
#undef FALSE
#undef ABS
#undef MIN
#undef MAX
#undef SIGN
#undef SWAP
#undef ABS_OF_EXPR
#undef integer_t
#undef real_t
#undef complex_t
#undef ZERO
#undef ONE
#undef TWO
#undef FABS
#undef SQRT
#undef LOG10
#undef GPL_NUM_ADD
#undef GPL_NUM_SUB
#undef GPL_NUM_MUL
#undef GPL_NUM_DIV
#undef GPL_NUM_EQ
#undef GPL_NUM_NE
#undef GPL_NUM_GE
#undef GPL_NUM_GT
#undef GPL_NUM_LE
#undef GPL_NUM_LT
#undef GPL_NUM_CMP
#undef GPL_NUM_STORE
#undef SLAMCH
#undef I1MACH
#undef R1MACH
#undef SASUM
#undef SCASUM
#undef ISAMAX
#undef ICAMAX
#undef SAXPY
#undef SCOPY
#undef SDOT
#undef SDSDOT
#undef DSDOT
#undef ENORM
#undef SNRM2
#undef SROT
#undef SROTG
#undef SSCAL
#undef SSWAP
#undef SGEFA
#undef SGECO
#undef SGEDI
#undef SGESL
#undef SGEFS
#undef SGEIR
#undef SPOCO
#undef SPOFA
#undef SPOFS
#undef SPOSL
#undef SQRDC
#undef SQRSL
#undef QRFAC
#undef RWUPDT
#undef QRSOLV
#undef SGTSL
#undef SSVDC
#undef SSVDC
#undef SSVCHK
#undef SSVSLV

#define TRUE  1
#define FALSE 0
#define ABS(a)    ((a)>=0?(a):-(a))
#define MIN(a,b)  ((a)<=(b)?(a):(b))
#define MAX(a,b)  ((a)>=(b)?(a):(b))
#define SIGN(a,b) ((b)>=0?((a)>=0?(a):-(a)):((a)<=0?(a):-(a)))
#define ABS_OF_EXPR(tmpvar,expr)    (tmpvar= (expr), ABS(tmpvar))
#define SWAP3(tmpvar, var1, var2)    tmpvar=var1; var1=var2; var2=tmpvar
#define SDSDOT(n,b,x,incx,y,incy)   (real_t)(DSDOT(n,x,incx,y,incy)+(b))

#define integer_t    int /* maybe `long' on your machine */
#define I1MACH   gpl_num_i1mach

#ifdef GPL_NUM_SINGLE_PRECISION
# define real_t      float
# define complex_t   gpl_fcomplex_t
# if USE_CONST_VAR_FOR_CONSTANTS
static const float zero_f= 0.0f, one_f= 1.0f, two_f= 2.0f;
#  define ZERO       zero_f
#  define ONE        one_f
#  define TWO        two_f
# else
#  define ZERO       0.0f
#  define ONE        1.0f
#  define TWO        2.0f
# endif
# define SQRT        sqrt
# define LOG10       log10
# define FABS        fabs
# define GPL_NUM_ADD      gpl_num_add_f
# define GPL_NUM_SUB      gpl_num_sub_f
# define GPL_NUM_MUL      gpl_num_mul_f
# define GPL_NUM_DIV      gpl_num_div_f
# define GPL_NUM_EQ       gpl_num_eq_f
# define GPL_NUM_NE       gpl_num_ne_f
# define GPL_NUM_GE       gpl_num_ge_f
# define GPL_NUM_GT       gpl_num_gt_f
# define GPL_NUM_LE       gpl_num_le_f
# define GPL_NUM_LT       gpl_num_lt_f
# define GPL_NUM_CMP      gpl_num_cmp_f
# define GPL_NUM_STORE    gpl_num_store_f
# define SLAMCH      gpl_num_slamch
# define R1MACH      gpl_num_r1mach
# define SASUM       gpl_num_asum_f
# define SCASUM      gpl_num_asum_c
# define ISAMAX      gpl_num_iamax_f
# define ICAMAX      gpl_num_iamax_c
# define SAXPY       gpl_num_axpy_f
# define SCOPY       gpl_num_copy_f
# define DSDOT       gpl_num_ddot_f
# define SDOT        gpl_num_dot_f
# define ENORM       gpl_num_enorm_f
# define SNRM2       gpl_num_nrm2_f
# define SROT        gpl_num_rot_f
# define SROTG       gpl_num_rotg_f
# define SSCAL       gpl_num_scal_f
# define SSWAP       gpl_num_swap_f
# define SGTSL       gpl_num_gtsl_f
# define SGEFA       gpl_num_gefa_f
# define SGECO       gpl_num_geco_f
# define SGEDI       gpl_num_gedi_f
# define SGESL       gpl_num_gesl_f
# define SGEFS       gpl_num_gefs_f
# define SGEIR       gpl_num_geir_f
# define SPOCO       gpl_num_poco_f
# define SPOFS       gpl_num_pofs_f
# define SPOFA       gpl_num_pofa_f
# define SPOSL       gpl_num_posl_f
# define SQRDC       gpl_num_qrdc_f
# define SQRSL       gpl_num_qrsl_f
# define QRFAC       gpl_num_qrfac_f
# define RWUPDT      gpl_num_rwupdt_f
# define QRSOLV      gpl_num_qrsolv_f
# define SSVDC       gpl_num_svdc_f
# define SSVCHK      gpl_num_svchk_f
# define SSVSLV      gpl_num_svslv_f
#endif /* GPL_NUM_SINGLE_PRECISION */

#ifdef GPL_NUM_DOUBLE_PRECISION
# define real_t      double
# define complex_t   gpl_dcomplex_t
# if USE_CONST_VAR_FOR_CONSTANTS
static const float zero_d= 0.0, one_d= 1.0, two_d= 2.0;
#  define ZERO       zero_d
#  define ONE        one_d
#  define TWO        two_d
# else
#  define ZERO       0.0
#  define ONE        1.0
#  define TWO        2.0
# endif
# define SQRT        sqrt
# define LOG10       log10
# define FABS        fabs
# define GPL_NUM_ADD      gpl_num_add_d
# define GPL_NUM_SUB      gpl_num_sub_d
# define GPL_NUM_MUL      gpl_num_mul_d
# define GPL_NUM_DIV      gpl_num_div_d
# define GPL_NUM_EQ       gpl_num_eq_d
# define GPL_NUM_NE       gpl_num_ne_d
# define GPL_NUM_GE       gpl_num_ge_d
# define GPL_NUM_GT       gpl_num_gt_d
# define GPL_NUM_LE       gpl_num_le_d
# define GPL_NUM_LT       gpl_num_lt_d
# define GPL_NUM_CMP      gpl_num_cmp_d
# define GPL_NUM_STORE    gpl_num_store_d
# define SLAMCH      gpl_num_dlamch
# define R1MACH      gpl_num_r1mach
# define SASUM       gpl_num_asum_d
# define SCASUM      gpl_num_asum_z
# define ISAMAX      gpl_num_iamax_d
# define ICAMAX      gpl_num_iamax_z
# define SAXPY       gpl_num_axpy_d
# define SCOPY       gpl_num_copy_d
# define DSDOT       gpl_num_dot_d
# define SDOT        gpl_num_dot_d
# define ENORM       gpl_num_enorm_d
# define SNRM2       gpl_num_nrm2_d
# define SROT        gpl_num_rot_d
# define SROTG       gpl_num_rotg_d
# define SSCAL       gpl_num_scal_d
# define SSWAP       gpl_num_swap_d
# define SGTSL       gpl_num_gtsl_d
# define SGEFA       gpl_num_gefa_d
# define SGECO       gpl_num_geco_d
# define SGEDI       gpl_num_gedi_d
# define SGESL       gpl_num_gesl_d
# define SGEFS       gpl_num_gefs_d
# define SGEIR       gpl_num_geir_d
# define SPOCO       gpl_num_poco_d
# define SPOFS       gpl_num_pofs_d
# define SPOFA       gpl_num_pofa_d
# define SPOSL       gpl_num_posl_d
# define SQRDC       gpl_num_qrdc_d
# define SQRSL       gpl_num_qrsl_d
# define QRFAC       gpl_num_qrfac_d
# define RWUPDT      gpl_num_rwupdt_d
# define QRSOLV      gpl_num_qrsolv_d
# define SSVDC       gpl_num_svdc_d
# define SSVCHK      gpl_num_svchk_d
# define SSVSLV      gpl_num_svslv_d
#endif /* GPL_NUM_DOUBLE_PRECISION */
