/*
 * gpl_ops2.h --
 *
 *	Definitions for Matrix oriented routines in GPLib library.
 *
 * Copyright (c) 1999 Eric THIEBAUT.
 *	See the file "LICENSE" for information on usage and redistribution
 *	of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * History:
 *	$Id$
 *	$Log$
 */
#ifndef GPL_OPS2_H
#define GPL_OPS2_H 1

#include "gpl_base.h"

#ifdef __cplusplus
extern "C" {
#endif

/*---------------------------------------------------------------------------*/
/* MATRIX TRANSPOSE */

extern gpl_byte_t     *gpl_transpose_b(gpl_byte_t *dst, const gpl_byte_t *src,
				   int src_dim1, int src_dim2);
extern short        *gpl_transpose_s(short *dst, const short *src,
				   int src_dim1, int src_dim2);
extern int          *gpl_transpose_i(int *dst, const int *src,
				   int src_dim1, int src_dim2);
extern long         *gpl_transpose_l(long *dst, const long *src,
				   int src_dim1, int src_dim2);
extern float        *gpl_transpose_f(float *dst, const float *src,
				   int src_dim1, int src_dim2);
extern double       *gpl_transpose_d(double *dst, const double *src,
				   int src_dim1, int src_dim2);
extern gpl_fcomplex_t *gpl_transpose_c(gpl_fcomplex_t *dst, const gpl_fcomplex_t *src,
				   int src_dim1, int src_dim2);
extern gpl_dcomplex_t *gpl_transpose_z(gpl_dcomplex_t *dst, const gpl_dcomplex_t *src,
				   int src_dim1, int src_dim2);
/*	Transpose matrix SRC and store the result in DST (must be different
	from SRC) which is returned.  Dimensions are SRC_DIM1-by-SRC_DIM2
	on input and SRC_DIM2-by-SRC_DIM1 on ouput. */

/*---------------------------------------------------------------------------*/
/* MATRIX MULTIPLICATION */

extern gpl_byte_t     *gpl_mmlt_b(gpl_byte_t dst[], const gpl_byte_t lop[],
			      const gpl_byte_t rop[],
			      int ni, int nj, int nk, int transpose);
extern short        *gpl_mmlt_s(short dst[], const short lop[],
			      const short rop[],
			      int ni, int nj, int nk, int transpose);
extern int          *gpl_mmlt_i(int dst[], const int lop[],
			      const int rop[],
			      int ni, int nj, int nk, int transpose);
extern long         *gpl_mmlt_l(long dst[], const long lop[],
			      const long rop[],
			      int ni, int nj, int nk, int transpose);
extern float        *gpl_mmlt_f(float dst[], const float lop[],
			      const float rop[],
			      int ni, int nj, int nk, int transpose);
extern double       *gpl_mmlt_d(double dst[], const double lop[],
			      const double rop[],
			      int ni, int nj, int nk, int transpose);
extern gpl_fcomplex_t *gpl_mmlt_c(gpl_fcomplex_t dst[],
			      const gpl_fcomplex_t lop[],
			      const gpl_fcomplex_t rop[],
			      int ni, int nj, int nk, int transpose);
extern gpl_dcomplex_t *gpl_mmlt_z(gpl_dcomplex_t dst[],
			      const gpl_dcomplex_t lop[],
			      const gpl_dcomplex_t rop[],
			      int ni, int nj, int nk, int transpose);
/*	Store in NI-by-NJ destination array DST the matrix product of left
	and right operands LOP and ROP.  NK is the number of elements along
	the summation dimension.  TRANSPOSE is a bit mask: (TRANSPOSE&1)
	and (TRANSPOSE&2) indicate whether or not the left (respectively
	right) operand should be transposed.  The following table summarizes
	the inputs/ouputs of the routines (note: the actual order of
	operations is optimized and depends on NI,NJ,NK):
	+-----------------+-----------------------------+---------+---------+
	| TRANSPOSE       | DST[i+NI*j]=                | dim. of | dim. of |
	|     |  RESULT   |        \sum_{k=0..NK-1} ... |   ROP   |   LOP   |
	+-----+-----------+-----------------------------+---------+---------+
	|  0  | LOP .ROP  |  LOP[i+NI*k] * ROP[k+NK*j]  | (NI,NK) | (NK,NJ) |
	|  1  | LOP'.LOP  |  LOP[k+NK*i] * ROP[k+NK*j]  | (NK,NI) | (NK,NJ) |
	|  2  | LOP .ROP' |  LOP[i+NI*k] * ROP[j+NJ*k]  | (NI,NK) | (NJ,NK) |
	|  3  | LOP'.ROP' |  LOP[k+NK*i] * ROP[j+NJ*k]  | (NK,NI) | (NJ,NK) |
	+-----+-----------+-----------------------------+---------+---------+
	Where: i=0..NI-1, j=0..NJ-1, k=0..NK-1 and the dimensions are given
	in the order of the faster varying one to the slowest varying one.
	Each routine return DST. */
/*---------------------------------------------------------------------------*/
#ifdef __cplusplus
} /* C++ */
#endif
#endif /* GPL_OPS2_H ----------------------------------------------------------*/
