/*
 * gpl_timer.c --
 *
 *	Timer routines for GPLib library.  Based on code from Yorick by
 *	David Munro.
 *
 * Copyright and warranties:
 *	Copyright (c) 1999 Eric THIEBAUT.
 *	
 *	This program is  free  software;  you  can  redistribute  it and/or
 *	modify it under the terms of  the  GNU  General  Public  License as
 *	published by the Free Software Foundation; either version 2  of the
 *	License, or (at your option) any later version.
 *	
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT  ANY  WARRANTY;  without  even  the  implied   warranty  of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.   See  the GNU
 *	General Public License for more details.
 *	
 *	You should have received a copy of the GNU  General  Public License
 *	along with this program (see file ``LICENSE'' at the  top directory
 *	of  this  distribution);  if  not,  write  to  the   Free  Software
 *	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *	
 *	To contact me, mail at: Eric THIEBAUT
 *				Observatoire de Lyon
 *				9, avenue Charles Andre
 *				F-69561 Saint Genis Laval Cedex
 *	or e-mail at:           thiebaut@obs.univ-lyon1.fr
 *
 * History:
 *	$Id$
 *	$Log$
 */

#include "gpl_base.h"

static double init_wall= -1.0e31;

#ifdef NO_TIMER
#ifndef CLOCK_FUNCTION
#define CLOCK_FUNCTION clock
#endif
#endif

#ifndef CLOCK_FUNCTION
#ifdef BSD_TIMER
/* This will work on our Suns -- other BSD machines may differ?
    sys/time.h includes time.h on Sun */
#include <sys/time.h>
#include <sys/resource.h>
void gpl_timer(double *cpu, double *sys, double *wall)
{
  struct rusage cpu_time;
  struct timeval wall_time;
  struct timezone zone;
  getrusage(RUSAGE_SELF, &cpu_time);
  *cpu= cpu_time.ru_utime.tv_sec + 1.0e-6*cpu_time.ru_utime.tv_usec;
  *sys= cpu_time.ru_stime.tv_sec + 1.0e-6*cpu_time.ru_stime.tv_usec;
  gettimeofday(&wall_time, &zone);
  *wall= wall_time.tv_sec + 1.0e-6*wall_time.tv_usec;
  if (init_wall<-1.0e30) init_wall= *wall;
  *wall-= init_wall;
}

#else
/* Assume POSIX 1003.1-1990 standard timing interface.
   However-- CLK_TCK is noted as "obsolescent" there...  */
#include <time.h>
#include <sys/times.h>
/* Try to handle modest deviations from POSIX standard (e.g.- Sun).  */
#ifndef CLK_TCK
#include <unistd.h>
#ifndef CLK_TCK
#define CLK_TCK sysconf(_SC_CLK_TCK)
#endif
#endif
static double ticks_per_second= 0.0;
void gpl_timer(double *cpu, double *sys, double *wall)
{
  struct tms cpu_time;
  long wall_ticks= times(&cpu_time);
  if (ticks_per_second==0.0) ticks_per_second= CLK_TCK;
  *cpu= cpu_time.tms_utime/(double)ticks_per_second;
  *sys= cpu_time.tms_stime/(double)ticks_per_second;
  *wall= wall_ticks/(double)ticks_per_second;
  if (init_wall<-1.0e30) init_wall= *wall;
  *wall-= init_wall;
}

#endif
#else
/* Define CLOCK_FUNCTION then modify the following version to suit
   your architecture.  See sysdep.h for other suggestions. */
#include <time.h>

void gpl_timer(double *cpu, double *sys, double *wall)
{
  *cpu= *wall= ((double)CLOCK_FUNCTION()) / ((double)TICKS_PER_SEC);
  *sys= 0.0;
  if (init_wall<-1.0e30) init_wall= *wall;
  *wall-= init_wall;
}
#endif
