/*                                                                     -*-C-*-
 * gpl_xdr.h --
 *
 *	Definitions for XDR (eXternal Data Representation) input/output
 *	routines for GPLib.
 *
 * Warning:
 *	This package may not be portable since it assumes that at most byte
 *	swapping is needed for your machine data layout to match XDR (works
 *	on most IEEE machines: PC's, Sun, IBM or HP workstations, ...).
 *	You may also have to modify some typedefs...
 *
 * Copyright (c) 1999 Eric THIEBAUT.
 *	See the file "LICENSE" for information on usage and redistribution
 *	of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * History:
 *	$Id$
 *	$Log$
 *-----------------------------------------------------------------------------
 */
#ifndef _GPL_XDR_H
#define _GPL_XDR_H 1

#include "gpl_base.h"

GPL_BEGIN_DECLS

/*---------------------------------------------------------------------------*/
/* Definitions of primary XDR data types. */
typedef unsigned char	gpl_xdr_byte_t;
typedef unsigned char	gpl_xdr_uchar_t;
typedef	         char	gpl_xdr_char_t;
typedef	gpl_int16_t	gpl_xdr_short_t;
typedef	gpl_uint16_t	gpl_xdr_ushort_t;
typedef	gpl_int32_t	gpl_xdr_int_t;
typedef	gpl_uint32_t	gpl_xdr_uint_t;
typedef	gpl_int32_t	gpl_xdr_long_t;
typedef	gpl_uint32_t	gpl_xdr_ulong_t;
typedef	float		gpl_xdr_float_t;
typedef	double		gpl_xdr_double_t;

/* Sizes (in bytes) of primary XDR data types. */
#define GPL_XDR_SIZEOF_BYTE   1
#define GPL_XDR_SIZEOF_CHAR   1
#define GPL_XDR_SIZEOF_UCHAR  GPL_XDR_SIZEOF_CHAR
#define GPL_XDR_SIZEOF_SHORT  2
#define GPL_XDR_SIZEOF_USHORT GPL_XDR_SIZEOF_SHORT
#define GPL_XDR_SIZEOF_INT    4
#define GPL_XDR_SIZEOF_UINT   GPL_XDR_SIZEOF_INT
#define GPL_XDR_SIZEOF_LONG   4
#define GPL_XDR_SIZEOF_ULONG  GPL_XDR_SIZEOF_LONG
#define GPL_XDR_SIZEOF_FLOAT  4
#define GPL_XDR_SIZEOF_DOUBLE 8

/* Routines to read/write XDR data. Behave like fread/fwrite but also take
   care of byte swapping data.  PTR must point to, at least, NUMBER elements
   of corresponding type. */
extern int gpl_xdr_read_byte(void *ptr, int number, FILE *stream);
extern int gpl_xdr_write_byte(const void *ptr, int number, FILE *stream);
extern int gpl_xdr_read_short(void *ptr, int number, FILE *stream);
extern int gpl_xdr_write_short(const void *ptr, int number, FILE *stream);
extern int gpl_xdr_read_int(void *ptr, int number, FILE *stream);
extern int gpl_xdr_write_int(const void *ptr, int number, FILE *stream);
extern int gpl_xdr_read_long(void *ptr, int number, FILE *stream);
extern int gpl_xdr_write_long(const void *ptr, int number, FILE *stream);
extern int gpl_xdr_read_float(void *ptr, int number, FILE *stream);
extern int gpl_xdr_write_float(const void *ptr, int number, FILE *stream);
extern int gpl_xdr_read_double(void *ptr, int number, FILE *stream);
extern int gpl_xdr_write_double(const void *ptr, int number, FILE *stream);

/* Direct call for bytes. */
#define gpl_xdr_read_byte(ptr, number, stream)  fread(ptr, 1, number, stream)
#define gpl_xdr_write_byte(ptr, number, stream) fwrite(ptr, 1, number, stream)

/* In XDR, int's and long's are identical as well as char and byte: */
#define gpl_xdr_read_int   gpl_xdr_read_long
#define gpl_xdr_write_int  gpl_xdr_write_long
#define gpl_xdr_read_char  gpl_xdr_read_byte
#define gpl_xdr_write_char gpl_xdr_write_byte

/*---------------------------------------------------------------------------*/
GPL_END_DECLS
#endif /* _GPL_XDR_H */
