/* 
 * tclsrv_main.c --
 *
 *	Provides a default version of the main program and Tcl_AppInit
 *	procedure for Tcl/Tk with TclSrv extension applications.
 *
 *-----------------------------------------------------------------------------
 *
 *	Copyright (C) 2002 Eric THIEBAUT.
 *
 *	This file is part of TclSrv.
 *
 *	TclSrv is free  software; you can redistribute it  and/or modify it
 *	under the terms  of the GNU General Public  License as published by
 *	the Free Software  Foundation; either version 2 of  the License, or
 *	(at your option) any later version.
 *
 *	TclSrv  is distributed  in the  hope that  it will  be  useful, but
 *	WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 *	MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 *	General Public License for more details.
 *
 *	You should have  received a copy of the  GNU General Public License
 *	along with TclSrv (file "COPYING"  in the top source directory); if
 *	not, write to the Free  Software Foundation, Inc., 59 Temple Place,
 *	Suite 330, Boston, MA 02111-1307 USA.
 *
 *-----------------------------------------------------------------------------
 *
 * History:
 *	$Id$
 *	$Log$
 */


/* Figure out which Tcl extensions to use. */
#ifndef WITH_TK
# define WITH_TK 0
#endif

/* Common Tcl extensions. */
#ifndef WITH_TCLX
# define WITH_TCLX 0
#endif
#ifndef WITH_EXPECT
# define WITH_EXPECT 0
#endif
#ifndef WITH_ITCL
# define WITH_ITCL 0
#endif

/* Common Tk extensions (unused if !WITH_TK). */
#ifndef WITH_TIX
# define WITH_TIX 0
#endif
#ifndef WITH_BLT
# define WITH_BLT 0
#endif
#ifndef WITH_TKPGPLOT
# define WITH_TKPGPLOT 0
#endif

/* Include headers. */
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <tcl.h>
#if WITH_EXPECT
# include <expect.h>
#endif
#if WITH_ITCL
# include <itcl.h>
#endif
#if WITH_TK
# include <tk.h>
# if WITH_ITCL
#  ifndef ITCL_MAJOR_VERSION
#   define ITCL_MAJOR_VERSION 1
#  endif
#  if ITCL_MAJOR_VERSION >= 2
#   include <itk.h>
#  endif
# endif /* WITH_ITCL */
# if WITH_BLT
#  include <blt.h>
# endif
# if WITH_TIX
#  include <tix.h>
# endif
# if WITH_TKPGPLOT
#  include "tkpgplot.h"
# endif
#endif /* WITH_TK */
#include "tclsrv.h"

/* Initialization file. */
#ifndef RC_FILE_NAME
# if WITH_TK
#   define RC_FILE_NAME "~/.wishrc"
# else
#   define RC_FILE_NAME "~/.tclshrc"
# endif
#endif


#if TCL_MAJOR_VERSION < 7 || TCL_MAJOR_VERSION == 7 && TCL_MINOR_VERSION < 4
/* The following variable is a special hack that allows applications
   to be linked using the procedure "main" from the Tcl library.  The
   variable generates a reference to "main", which causes main to be
   brought in from the library (and all of Tcl with it). */
extern int main(void);
int *tclDummyMainPtr = (int *)main;
#else
/* The following variable is a special hack that is needed in order for
   Sun shared libraries to be used for Tcl. */
extern int matherr(void);
int *tclDummyMathPtr = (int *) matherr;
/*
 *----------------------------------------------------------------------
 *
 * main --
 *
 *	This is the main program for the application.
 *
 * Results:
 *	None: Tcl_Main never returns here, so this procedure never
 *	returns either.
 *
 * Side effects:
 *	Whatever the application does.
 *
 *----------------------------------------------------------------------
 */

static CONST char *server_path=NULL; 
extern char *strdup _ANSI_ARGS_((CONST char *s));

int main(int argc, char **argv)
{
  int i;
  for (i=1 ; i<argc ; ++i) {
    char *arg = argv[i];
    if (arg[0] != '-' || (arg[1] == '-' && arg[2] == 0)) break;
    if (arg[1] == 's' && strcmp(arg, "-server") == 0) {
      if (++i >= argc) {
	fprintf(stderr, "%s: option \"-server\" requires one argument.\n",
		argv[0]);
	exit(1);
      }
      server_path = strdup(argv[i]);
      while (++i < argc) argv[i-2] = argv[i];
      argc -= 2;
    }
  }
#if WITH_TK
  Tk_Main(argc, argv, Tcl_AppInit);
#else
  Tcl_Main(argc, argv, Tcl_AppInit);
#endif
  return 0; /* Needed only to prevent compiler warning. */
}

#endif

/*
 *----------------------------------------------------------------------
 *
 * Tcl_AppInit --
 *
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

int Tcl_AppInit(interp)
     Tcl_Interp *interp;
{
  /*
   * Call the init procedures for included packages.  Each call should
   * look like this:
   *
   * if (Mod_Init(interp) == TCL_ERROR) return TCL_ERROR;
   *
   * where "Mod" is the name of the module.
   */

  if (Tcl_Init(interp) == TCL_ERROR) return TCL_ERROR;

#if WITH_TK
  if (Tk_Init(interp) == TCL_ERROR) return TCL_ERROR;
#endif

#if WITH_ITCL
  if (Itcl_Init(interp) == TCL_ERROR) return TCL_ERROR;
# if WITH_TK && ITCL_MAJOR_VERSION >= 2
  if (Itk_Init(interp) == TCL_ERROR) return TCL_ERROR;
# endif
#endif /* WITH_ITCL */


#if WITH_TK
# if WITH_TIX
  if (Tix_Init(interp) == TCL_ERROR) return TCL_ERROR;
# endif
# if WITH_BLT
  if (Blt_Init(interp) == TCL_ERROR) return TCL_ERROR;
# endif
# if WITH_TKPGPLOT
  if (Tkpgplot_Init(interp) == TCL_ERROR) return TCL_ERROR;
# endif
#endif /* WITH_TK */

#if WITH_EXPECT
  if (Exp_Init(interp) == TCL_ERROR) return TCL_ERROR;
#endif

  if (Tclsrv_Init(interp) == TCL_ERROR) return TCL_ERROR;
  if (server_path &&
      Tclsrv_CreateCommandServer(interp, server_path,
				 TCLSRV_MODE_PRIVATE) != TCL_OK) {
    exit(1);
  }


  /* Call Tcl_CreateCommand for application-specific commands, if they
     weren't already created by the init procedures called above. */

  /* Specify a user-specific startup file to invoke if the application is run
     interactively.  Typically the startup file is "~/.apprc" where "app" is
     the name of the application.  If this line is deleted then no
     user-specific startup file will be run under any conditions. */
#if TCL_MAJOR_VERSION>= 8 || (TCL_MAJOR_VERSION==7 && TCL_MINOR_VERSION>4) 
  Tcl_SetVar(interp, "tcl_rcFileName", RC_FILE_NAME, TCL_GLOBAL_ONLY);
#else
  tcl_RcFileName= RC_FILE_NAME;
#endif
  return TCL_OK;
}
