/*
 * tclsrv_private.h --
 *
 *	Private definitions for TclSrv.
 *
 *-----------------------------------------------------------------------------
 *
 *	Copyright (C) 2002 Eric THIEBAUT.
 *
 *	This file is part of TclSrv.
 *
 *	TclSrv is free  software; you can redistribute it  and/or modify it
 *	under the terms  of the GNU General Public  License as published by
 *	the Free Software  Foundation; either version 2 of  the License, or
 *	(at your option) any later version.
 *
 *	TclSrv  is distributed  in the  hope that  it will  be  useful, but
 *	WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 *	MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 *	General Public License for more details.
 *
 *	You should have  received a copy of the  GNU General Public License
 *	along with TclSrv (file "COPYING"  in the top source directory); if
 *	not, write to the Free  Software Foundation, Inc., 59 Temple Place,
 *	Suite 330, Boston, MA 02111-1307 USA.
 *
 *-----------------------------------------------------------------------------
 *
 * History:
 *	$Id$
 *	$Log$
 */

#ifndef _TCLSRV_PRIVATE_H
#define _TCLSRV_PRIVATE_H 1

#include "tclsrv.h"

#if (! defined(HAVE_INLINE))
#  if (defined(__GNUC__) && __GNUC__ > 1)
#    define HAVE_INLINE 1
#  else
#    define HAVE_INLINE 0
#  endif
#endif

#if HAVE_INLINE
#  define TCLSRV_INLINE inline
#else
#  define TCLSRV_INLINE static
#endif


/* Figure out an integer type at least 4 bytes long.  */
#include <limits.h>
#if (defined(INT_MAX) && INT_MAX >= 2147483647L)
typedef unsigned int tclsrv_uint_t;
#else
# if (defined(LONG_MAX) && LONG_MAX >= 2147483647L)
typedef unsigned long tclsrv_uint_t;
# else
#  error "no suitable integer type found"
# endif
#endif


/* Socket i/o buffer size to improve data transfert (set it to 0 to do not
   change size of default buffers and therefore not use getsockopt nor
   setsockopt functions, e.g. if you don't have them). */
#ifndef SOCKET_BUFSIZE
#  define SOCKET_BUFSIZE 4096
#endif


/* Magic bytes used in message header. */
#define TCLSRV_MAGIC_0 0xDE
#define TCLSRV_MAGIC_1 0xAD
#define TCLSRV_MAGIC_2 0xC0
#define TCLSRV_MAGIC_3 0xDE

/* Macros to encode/decode header of messages (H must be an unsigned char
   array/pointer and length of message L must be a size_t value). */
#define BAD_MAGIC(h) ((h)[0] != TCLSRV_MAGIC_0 || (h)[1] != TCLSRV_MAGIC_1 || \
                      (h)[2] != TCLSRV_MAGIC_2 || (h)[3] != TCLSRV_MAGIC_3)
#define SET_MAGIC(h)  (h)[0] = TCLSRV_MAGIC_0; \
                      (h)[1] = TCLSRV_MAGIC_1; \
                      (h)[2] = TCLSRV_MAGIC_2; \
                      (h)[3] = TCLSRV_MAGIC_3
#define SET_LENGTH(h,l) (h)[4] = ((l)>>24) & 0xff; \
                        (h)[5] = ((l)>>16) & 0xff; \
                        (h)[6] = ((l)>> 8) & 0xff; \
                        (h)[7] =  (l)      & 0xff
#define GET_LENGTH(h) ((((size_t)(h)[4])<<24) | (((size_t)(h)[5])<<16) | \
	               (((size_t)(h)[6])<< 8) |  ((size_t)(h)[7]))


/* Provide convenience macros for handling structure fields
   through their offsets. */
#define STRUCT_OFFSET(struct_type, member)	\
    ((size_t)((char *)&(((struct_type *)0)->member)))
#define STRUCT_MEMBER_P(struct_p, struct_offset)   \
    ((VOID *) ((char *)(struct_p) + (size_t)(struct_offset)))
#define STRUCT_MEMBER(member_type, struct_p, struct_offset)   \
    (*(member_type *)G_STRUCT_MEMBER_P((struct_p), (struct_offset)))

#endif /* _TCLSRV_PRIVATE_H -------------------------------------------------*/
